#include <gst/gst.h>
#include <string.h>
#include "gst_factory_lists.h"

/*******************************/
/* Functions declaration       */
/*******************************/

static gboolean gst_factory_list_is_type(GstElementFactory * factory, 
					 GstFactoryListType type);
static gboolean decoders_filter (GstElementFactory * factory);
static gboolean sinks_filter (GstElementFactory * factory);

/*******************************/
/* Implementation of functions */
/*******************************/

/* the filter function for selecting the elements we can use in
 * autoplugging */
gboolean decoders_filter (GstElementFactory * factory)
{
  guint rank;
  const gchar *klass;

  klass = gst_element_factory_get_klass (factory);
  /* only demuxers, decoders, depayloaders and parsers can play */
  if (strstr (klass, "Demux") == NULL &&
      strstr (klass, "Decoder") == NULL &&
      strstr (klass, "Depayloader") == NULL &&
      strstr (klass, "Parse") == NULL) {
    return FALSE;
  }

  /* only select elements with autoplugging rank */
  rank = gst_plugin_feature_get_rank (GST_PLUGIN_FEATURE (factory));
  if (rank < GST_RANK_MARGINAL)
    return FALSE;

  return TRUE;
}

/* the filter function for selecting the elements we can use in
 * autoplugging */
gboolean sinks_filter (GstElementFactory * factory)
{
  guint rank;
  const gchar *klass;

  klass = gst_element_factory_get_klass (factory);
  /* only sinks can play */
  if (strstr (klass, "Sink") == NULL) {
    return FALSE;
  }

  /* must be audio or video sink */
  if (strstr (klass, "Audio") == NULL && strstr (klass, "Video") == NULL) {
    return FALSE;
  }

  /* only select elements with autoplugging rank */
  rank = gst_plugin_feature_get_rank (GST_PLUGIN_FEATURE (factory));
  if (rank < GST_RANK_MARGINAL)
    return FALSE;

  return TRUE;
}

gboolean element_filter (GstPluginFeature *feature, 
			 FilterData *data)

{
  gboolean res;

  /* we only care about element factories */
  if (!GST_IS_ELEMENT_FACTORY(feature))
    return FALSE;
  
  res = gst_factory_list_is_type(GST_ELEMENT_FACTORY (feature), data->type);

  return res;
}

gboolean gst_factory_list_is_type(GstElementFactory * factory, 
				  GstFactoryListType type)
{
  gboolean res = FALSE;

  if (!res && (type & GST_FACTORY_LIST_SINK))
    res = sinks_filter (factory);
  if (!res && (type & GST_FACTORY_LIST_DECODER))
    res = decoders_filter (factory);

  return res;
}
